﻿using UnityEngine;
using HarmonyLib;
using System.IO;
using System;
using System.Collections.Generic;

public class StatModifier : Mod
{
    static Dictionary<GameMode, ValueMemory> oldValues = null;
    private static float modiFValue = 1f;
    private static float modiWValue = 1f;
    public static bool isInWorld => UnityEngine.SceneManagement.SceneManager.GetActiveScene().name == Semih_Network.GameSceneName;
    public static float currentFoodModifier
    {
        get
        {
            return modiFValue;
        }
        set {
            modiFValue = value;
            foreach (SO_GameModeValue modeValue in Traverse.Create(typeof(GameModeValueManager)).Field("gameModeValues").GetValue<SO_GameModeValue[]>())
            {
                if (!oldValues.ContainsKey(modeValue.gameMode))
                    oldValues.Add(modeValue.gameMode, new ValueMemory(modeValue));
                modeValue.nourishmentVariables.foodDecrementRateMultiplier = oldValues[modeValue.gameMode].Food * modiFValue;
            }
        }

    }
    public static float currentWaterModifier
    {
        get
        {
            return modiWValue;
        }
        set
        {
            modiWValue = value;
            foreach (SO_GameModeValue modeValue in Traverse.Create(typeof(GameModeValueManager)).Field("gameModeValues").GetValue<SO_GameModeValue[]>())
            {
                if (!oldValues.ContainsKey(modeValue.gameMode))
                    oldValues.Add(modeValue.gameMode, new ValueMemory(modeValue));
                modeValue.nourishmentVariables.thirstDecrementRateMultiplier = oldValues[modeValue.gameMode].Water * modiWValue;
            }
        }

    }
    Harmony harmony;
    public static StatModifier Instance;
    public void Start()
    {
        Instance = this;
        harmony = new Harmony("com.aidanamite.StatModifier");
        harmony.PatchAll();
        oldValues = new Dictionary<GameMode, ValueMemory>();
        foreach (SO_GameModeValue modeValue in Traverse.Create(typeof(GameModeValueManager)).Field("gameModeValues").GetValue<SO_GameModeValue[]>())
            oldValues.Add(modeValue.gameMode, new ValueMemory(modeValue));
        Log("Mod has been loaded!");
    }

    [ConsoleCommand(name: "statmodifier", docs: "Changes or returns current stat modifier")]
    public static string MyCommand(string[] args)
    {
        if (args.Length == 0)
            return "Hunger: " + currentFoodModifier.ToString() + "\nThirst: " + currentWaterModifier.ToString();
        if (args.Length > 1) 
            return "Too many arguments";
        try
        {
            float tmp = float.Parse(args[0]);
            currentFoodModifier = tmp;
            currentWaterModifier = tmp;
            StoreSettings();
            return "Stat modifiers are now " + tmp.ToString();
        } catch
        {
            return "Failed to parse \"" + args[0] + "\" as a number";
        }
    }

    [ConsoleCommand(name: "statmodifier:food", docs: "Changes or returns current hunger stat modifier")]
    public static string MyCommand2(string[] args)
    {
        if (args.Length == 0)
            return currentFoodModifier.ToString();
        if (args.Length > 1)
            return "Too many arguments";
        try
        {
            currentFoodModifier = float.Parse(args[0]);
            StoreSettings();
            return "Hunger stat modifier is now " + currentFoodModifier.ToString();
        }
        catch
        {
            return "Failed to parse \"" + args[0] + "\" as a number";
        }
    }

    [ConsoleCommand(name: "statmodifier:water", docs: "Changes or returns current thirst stat modifier")]
    public static string MyCommand3(string[] args)
    {
        if (args.Length == 0)
            return currentWaterModifier.ToString();
        if (args.Length > 1)
            return "Too many arguments";
        try
        {
            currentWaterModifier = float.Parse(args[0]);
            StoreSettings();
            return "Thirst stat modifier is now " + currentWaterModifier.ToString();
        }
        catch
        {
            return "Failed to parse \"" + args[0] + "\" as a number";
        }
    }

    public override void WorldEvent_WorldLoaded() => ExtraSettingsAPI_SettingsClose();

    public void OnModUnload()
    {
        harmony.UnpatchAll();
        foreach (SO_GameModeValue modeValue in Traverse.Create(typeof(GameModeValueManager)).Field("gameModeValues").GetValue<SO_GameModeValue[]>())
        {
            if (!oldValues.ContainsKey(modeValue.gameMode))
                continue;
            modeValue.nourishmentVariables.foodDecrementRateMultiplier = oldValues[modeValue.gameMode].Food;
            modeValue.nourishmentVariables.thirstDecrementRateMultiplier = oldValues[modeValue.gameMode].Water;
        }
        Log("Mod has been unloaded!");
    }

    public static void Log(object message)
    {
        Debug.Log("[StatModifier]: " + message.ToString());
    }
    public void ExtraSettingsAPI_Load() => ExtraSettingsAPI_SettingsClose();

    public static void StoreSettings()
    {
        Instance.ExtraSettingsAPI_SetSliderValue("Hunger Modifier", currentFoodModifier);
        Instance.ExtraSettingsAPI_SetSliderValue("Thirst Modifier", currentWaterModifier);
    }

    public void ExtraSettingsAPI_SettingsClose()
    {
        if (UnityEngine.SceneManagement.SceneManager.GetActiveScene().name == Semih_Network.GameSceneName)
        {
            currentFoodModifier = ExtraSettingsAPI_GetSliderRealValue("Hunger Modifier");
            currentWaterModifier = ExtraSettingsAPI_GetSliderRealValue("Thirst Modifier");
        }
    }
    static Traverse ExtraSettingsAPI_Traverse;
    static bool ExtraSettingsAPI_Loaded = false;
    public float ExtraSettingsAPI_GetSliderRealValue(string SettingName)
    {
        if (ExtraSettingsAPI_Loaded)
            return ExtraSettingsAPI_Traverse.Method("getSliderRealValue", new object[] { this, SettingName }).GetValue<float>();
        return 0;
    }
    public void ExtraSettingsAPI_SetSliderValue(string SettingName, float value)
    {
        if (ExtraSettingsAPI_Loaded)
            ExtraSettingsAPI_Traverse.Method("setSliderValue", new object[] { this, SettingName, value }).GetValue();
    }
}

[HarmonyPatch]
public class Patch_StatLostPerSecond
{
    [HarmonyPatch(typeof(Stat_Consumable), "LostPerSecond", MethodType.Getter)]
    static void Postfix(Stat_Consumable __instance, ref float __result, float ___lostPerSecondDefault)
    {
        var s = __instance.GetComponentInParent<PlayerStats>();
        var flag = s ? s.stat_hunger.normalConsumable == __instance || s.stat_hunger.bonusConsumable == __instance : false;
        if (flag)
            __result = ___lostPerSecondDefault * GameModeValueManager.GetCurrentGameModeValue().nourishmentVariables.foodDecrementRateMultiplier;
    }
}

public struct ValueMemory
{
    public float Food;
    public float Water;
    public ValueMemory(SO_GameModeValue Mode)
    {
        Food = Mode.nourishmentVariables.foodDecrementRateMultiplier;
        Water = Mode.nourishmentVariables.thirstDecrementRateMultiplier;
    }
}